@@ -25,11 +25,12 @@ NULL
 #'   \linkS4class{MizerSim} object (which contains a `MizerParams` object).
 #' @param effort The effort of each fishing gear through time. See notes below.
 #' @param t_max The number of years the projection runs for. The default value is
-#'   100. However, this argument is ignored if an array is used for the
+#'   100. This argument is ignored if an array is used for the
 #'   `effort` argument. See notes below.
 #' @param dt Time step of the solver. The default value is 0.1.
 #' @param t_save The frequency with which the output is stored. The default
-#'   value is 1. Must be an integer multiple of dt.
+#'   value is 1. This argument is ignored if an array is used for the `effort`
+#'   argument. See notes below.
 #' @param t_start The the year of the start of the simulation. The simulation
 #'   will cover the period from `t_start` to \code{t_start + t_max}.
 #'   Defaults to 0. Ignored if an array is used for the `effort`
@@ -71,8 +72,8 @@ NULL
 #' used as the initial time for the simulation. Otherwise the initial time is
 #' set to the final time of the previous simulation if `object` is a 
 #' `MizerSim` object or to `t_start` otherwise. Also, if the effort is
-#' an array then the `t_max` argument is ignored and the maximum simulation
-#' time is the largest time of the effort array.
+#' an array then the `t_max` and `t_save` arguments are ignored and the
+#' simulation times will be taken from the effort array.
 #' 
 #' If the `object` argument is of class `MizerSim` then the initial
 #' values for the simulation are taken from the final values in the 
@@ -107,10 +108,10 @@ project <- function(object, effort,
                     initial_n, initial_n_pp,
                     append = TRUE,
                     progress_bar = TRUE, ...) {
-    validObject(object)
     
     # Set and check initial values ----
     if (is(object, "MizerSim")) {
+        validObject(object)
         params <- object@params
         no_t <- dim(object@n)[[1]]
         initial_n <- params@initial_n # Needed to get the right dimensions
@@ -120,7 +121,7 @@ project <- function(object, effort,
         initial_n_other <- object@n_other[no_t, ]
         t_start <- as.numeric(dimnames(object@n)[[1]][[no_t]])
     } else if (is(object, "MizerParams")) {
-        params <- object
+        params <- validParams(object)
         if (missing(initial_n))       initial_n <- params@initial_n
         if (missing(initial_n_pp)) initial_n_pp <- params@initial_n_pp
         initial_n_other <- params@initial_n_other
@@ -152,75 +153,32 @@ project <- function(object, effort,
     
     # Set effort array ----
     if (missing(effort)) effort <- params@initial_effort
-    # Do we need to create an effort array?
-    if (is.null(dim(effort))) { # effort is a vector
-        no_gears <- dim(params@catchability)[1]
-        if ((length(effort) > 1) & (length(effort) != no_gears)) {
-            stop("Effort vector must be the same length as the number of fishing gears\n")
-        }
-        # If more than 1 gear need to check that gear names match
-        gear_names <- dimnames(params@catchability)[[1]]
-        effort_gear_names <- names(effort)
-        if (length(effort) == 1 && is.null(effort_gear_names)) {
-            effort_gear_names <- gear_names
-        }
-        if (!all(gear_names %in% effort_gear_names)) {
-            stop("Gear names in the MizerParams object (", 
-                 paste(gear_names, collapse = ", "), 
-                 ") do not match those in the effort vector.")
-        }
+    if (is.null(dim(effort))) { # effort is a vector or scalar
         # Set up the effort array transposed so we can use the recycling rules
-        time_dimnames <- c(t_start, t_start + t_max)
-        effort <- t(array(effort, dim = c(no_gears, 2), 
-                          dimnames = list(gear = effort_gear_names, 
-                                          time = time_dimnames)))
+        # no point running a simulation with no saved results
+        if (t_max < t_save) {
+            t_save <- t_max
+        }
+        times <- seq(t_start, t_start + t_max, by = t_save)
+        effort <- validEffortVector(effort, params)
+        effort <- t(array(effort, 
+                          dim = c(length(effort), length(times)), 
+                          dimnames = list(gear = names(effort), 
+                                          time = times)))
+    } else {
+        effort <- validEffortArray(effort, params)
     }
     
-    # Check that number and names of gears in effort array is same as in 
-    # MizerParams object
-    no_gears <- dim(params@catchability)[1]
-    if (dim(effort)[2] != no_gears) {
-        stop("The number of gears in the effort array (length of the second dimension = ", 
-             dim(effort)[2], 
-             ") does not equal the number of gears in the MizerParams object (", 
-             no_gears, ").")
-    }
-    gear_names <- dimnames(params@catchability)[[1]]
-    if (!all(gear_names %in% dimnames(effort)[[2]])) {
-        stop("Gear names in the MizerParams object (", 
-             paste(gear_names, collapse = ", "), 
-             ") do not match those in the effort array.")
-    }
-    # Sort effort array to match order in MizerParams
-    effort <- effort[, gear_names, drop = FALSE]
-    
-    if (is.null(dimnames(effort)[[1]])) {
-        stop("The time dimname of the effort argument must be numeric.")
-    }
-    time_effort <- as.numeric(dimnames(effort)[[1]])
-    if (any(is.na(time_effort))) {
-        stop("The time dimname of the effort argument must be numeric.")
-    }
-    if (is.unsorted(time_effort)) {
-        stop("The time dimname of the effort argument should be increasing.")
-    }
-    t_start <- time_effort[[1]]
-    t_end <- time_effort[[length(time_effort)]]
-    t_max <- t_end - t_start
+    times <- as.numeric(dimnames(effort)[[1]])
     
     # Make the MizerSim object with the right size ----
     # We only save every t_save years
-    # no point running a simulation with no saved results
-    if (t_max < t_save) {
-        t_save <- t_max
-    }
-    # Divisibility test needs to be careful about machine rounding errors,
-    # see https://github.com/sizespectrum/mizer/pull/2
-    if ((t_save < dt) || !isTRUE(all.equal((t_save - round(t_save / dt) * dt), 0)))
-        stop("t_save must be a positive multiple of dt")
-    skip <- round(t_save/dt)
-    t_dimnames <- seq(t_start, t_end, by = t_save)
-    sim <- MizerSim(params, t_dimnames = t_dimnames)
+    sim <- MizerSim(params, t_dimnames = times)
+    # Set initial population and effort
+    sim@n[1, , ] <- initial_n 
+    sim@n_pp[1, ] <- initial_n_pp
+    sim@n_other[1, ] <- initial_n_other
+    sim@effort <- effort
     
     ## Initialise ----
     # get functions
@@ -232,40 +190,33 @@ project <- function(object, effort,
     if (is(progress_bar, "Progress")) {
         # We have been passed a shiny progress object
         progress_bar$set(message = "Running simulation", value = 0)
-        proginc <- 1/length(t_dimnames)
+        proginc <- 1/length(times)
     } else if (progress_bar == TRUE) {
         pb <- progress::progress_bar$new(
             format = "[:bar] :percent ETA: :eta",
-            total = length(t_dimnames), width = 60)
+            total = length(times), width = 60)
         pb$tick(0)
     }
     
-    t <- t_start  # keep track of time
-    i_save_time <- 2 # index into which to save the results
-    
-    # set up variables for keeping track of effort changes
-    i_effort <- 1
-    current_effort <- effort[1, ]
-    t_next_effort <- time_effort[[2]] - 1e-8  # to avoid rounding problems
-    
-    # Set initial population and effort
     n_list <- list(n = initial_n, n_pp = initial_n_pp, n_other = initial_n_other)
-    sim@n[1, , ] <- initial_n 
-    sim@n_pp[1, ] <- initial_n_pp
-    sim@n_other[1, ] <- initial_n_other
-    sim@effort[1, ] <- current_effort
+    t <- times[[1]]
     
     ## Loop over time ----
-    for (i in 2:length(t_dimnames)) {
-        
+    for (i in 2:length(times)) {
+        # number of time steps between saved times
+        steps <- round((times[[i]] - t) / dt)
+        # advance to next saved time
         n_list <- project_simple(
             params, n = n_list$n, n_pp = n_list$n_pp, n_other = n_list$n_other,
-            t = t, dt = dt, steps = skip, effort = current_effort,
+            t = t, dt = dt, steps = steps, 
+            effort = effort[i - 1, ],
             resource_dynamics_fn = resource_dynamics_fn,
             other_dynamics_fns = other_dynamics_fns,
             rates_fns = rates_fns, ...)
-        t <- t + t_save
-        
+        # Calculate start time for next iteration
+        # The reason we don't simply use the next entry in `times` is that
+        # those entries may not be separated by exact multiples of dt.
+        t <- t + steps * dt
         # Advance progress bar
         if (is(progress_bar, "Progress")) {
             progress_bar$inc(amount = proginc)
@@ -273,29 +224,18 @@ project <- function(object, effort,
             pb$tick()
         }
         
-        # Do we need new effort?
-        if (t >= t_next_effort) {
-            i_effort <- i_effort + 1
-            current_effort <- effort[i_effort, ]
-            # We have to be careful when we reach the final time
-            t_next_effort <- ifelse(i_effort < length(time_effort),
-                                    time_effort[[i_effort + 1]] - 1e-8,
-                                    t_max + 100)
-        }
-        
         # Store result
         sim@n[i, , ] <- n_list$n
         sim@n_pp[i, ] <- n_list$n_pp
         sim@n_other[i, ] <- n_list$n_other
-        sim@effort[i, ] <- current_effort
     }
     
     # append to previous simulation ----
     if (is(object, "MizerSim") && append) {
         no_t_old <- dim(object@n)[1]
-        no_t <- length(t_dimnames)
+        no_t <- length(times)
         new_t_dimnames <- c(as.numeric(dimnames(object@n)[[1]]),
-                            t_dimnames[2:no_t])
+                            times[2:no_t])
         new_sim <- MizerSim(params, t_dimnames = new_t_dimnames)
         old_indices <- 1:no_t_old
         new_indices <- seq(from = no_t_old + 1, length.out = no_t - 1)
@@ -317,8 +257,8 @@ project <- function(object, effort,
 #' This is an internal function used by the user-facing `project()` function.
 #' It is of potential interest only to mizer extension authors.
 #' 
-#' The function does not check its arguments because it is mean to be as fast
-#' as possible to allow it to be used in a loop. For example it is called in
+#' The function does not check its arguments because it is meant to be as fast
+#' as possible to allow it to be used in a loop. For example, it is called in
 #' `project()` once for every saved value. The function also does not save its
 #' intermediate results but only returns the result at time `t + dt * steps`.
 #' During this time it uses the constant fishing effort `effort`.
@@ -367,7 +307,7 @@ project <- function(object, effort,
 project_simple <- function(params, n, n_pp, n_other, t, dt, steps, 
                            effort, resource_dynamics_fn, other_dynamics_fns,
                            rates_fns, ...) {    
-    # Handy things
+    # Handy things ----
     no_sp <- nrow(params@species_params) # number of species
     no_w <- length(params@w) # number of fish size bins
     idx <- 2:no_w
@@ -379,16 +319,14 @@ project_simple <- function(params, n, n_pp, n_other, t, dt, steps,
     a <- matrix(0, nrow = no_sp, ncol = no_w)
     b <- matrix(0, nrow = no_sp, ncol = no_w)
     S <- matrix(0, nrow = no_sp, ncol = no_w)
-    
+
+    # Loop over time steps ----
     for (i_time in 1:steps) {
         r <- rates_fns$Rates(
             params, n = n, n_pp = n_pp, n_other = n_other,
             t = t, effort = effort, rates_fns = rates_fns, ...)
         
-        # Update time
-        t <- t + dt
-        
-        # Update other components
+        # * Update other components ----
         n_other_current <- n_other  # So that the resource dynamics can still 
         # use the current value
         for (component in names(params@other_dynamics)) {
@@ -406,25 +344,19 @@ project_simple <- function(params, n, n_pp, n_other, t, dt, steps,
                 )
         }
         
-        # Update resource
+        # * Update resource ----
         n_pp <- resource_dynamics_fn(params, n = n, n_pp = n_pp,
                                      n_other = n_other_current, rates = r,
                                      t = t, dt = dt, ...)
         
-        # Iterate species one time step forward:
+        # * Update species ----
         # a_{ij} = - g_i(w_{j-1}) / dw_j dt
         a[, idx] <- sweep(-r$e_growth[, idx - 1, drop = FALSE] * dt, 2,
                           params@dw[idx], "/")
         # b_{ij} = 1 + g_i(w_j) / dw_j dt + \mu_i(w_j) dt
-        b[, idx] <- 1 + sweep(r$e_growth[, idx, drop = FALSE] * dt, 2, 
-                              params@dw[idx], "/") +
-            r$mort[, idx, drop = FALSE] * dt
+        b[] <- 1 + sweep(r$e_growth * dt, 2, params@dw, "/") + r$mort * dt
         # S_{ij} <- N_i(w_j)
         S[,idx] <- n[, idx, drop = FALSE]
-        # Boundary condition upstream end (reproduction)
-        b[w_min_idx_array_ref] <- 1 + r$e_growth[w_min_idx_array_ref] * dt /
-            params@dw[params@w_min_idx] +
-            r$mort[w_min_idx_array_ref] * dt
         # Update first size group of n
         n[w_min_idx_array_ref] <-
             (n[w_min_idx_array_ref] + r$rdd * dt / 
@@ -439,52 +371,42 @@ project_simple <- function(params, n, n_pp, n_other, t, dt, steps,
         n <- inner_project_loop(no_sp = no_sp, no_w = no_w, n = n,
                                 A = a, B = b, S = S,
                                 w_min_idx = params@w_min_idx)
+        
+        # * Update time ----
+        t <- t + dt
     }
+    
     return(list(n = n, n_pp = n_pp, n_other = n_other, rates = r))
 }
 
-#' Calculate initial population abundances for the community populations
-#' 
-#' This function uses the model parameters and other parameters to calculate 
-#' initial population abundances for the community populations. These initial 
-#' abundances should be reasonable guesses at the equilibrium values. The 
-#' returned population can be passed to the `project` function.
-#' 
-#' @param params The model parameters. An object of type \linkS4class{MizerParams}.
-#' @param a A parameter with a default value of 0.35.
-#' @param n0_mult Multiplier for the abundance at size 0. Default value is
-#'   kappa/1000.
-#' @export
-#' @return A matrix (species x size) of population abundances.
-#' @examples
-#' \dontrun{
-#' params <- newMultispeciesParams(NS_species_params_gears)
-#' init_n <- get_initial_n(params)
-#' }
-get_initial_n <- function(params, n0_mult = NULL, a = 0.35) {
-    if (!is(params,"MizerParams"))
-        stop("params argument must of type MizerParams")
-    no_sp <- nrow(params@species_params)
-    no_w <- length(params@w)
-    initial_n <- array(NA, dim = c(no_sp, no_w))
-    dimnames(initial_n) <- dimnames(params@intake_max)
-    # N = N0 * Winf^(2*n-q-2+a) * w^(-n-a)
-    # Reverse calc n and q from intake_max and search_vol slots (could add get_n function)
-    n <- params@species_params$n[[1]]
-    q <- params@species_params$q[[1]]
-    # Guessing at a suitable n0 value based on kappa - this was figured out 
-    # using trial and error and should be updated
-    if (is.null(n0_mult)) {
-        lambda <- 2 + q - n
-        kappa <- params@cc_pp[1] / (params@w_full[1]^(-lambda))
-        n0_mult <- kappa / 1000
+validEffortArray <- function(effort, params) {
+    # Check that number and names of gears in effort array is same as in 
+    # MizerParams object
+    no_gears <- dim(params@catchability)[1]
+    if (dim(effort)[2] != no_gears) {
+        stop("The number of gears in the effort array (length of the second dimension = ", 
+             dim(effort)[2], 
+             ") does not equal the number of gears in the MizerParams object (", 
+             no_gears, ").")
     }
-    initial_n[] <- unlist(tapply(params@w, 1:no_w, function(wx,n0_mult,w_inf,a,n,q)
-        n0_mult * w_inf^(2 * n - q - 2 + a) * wx^(-n - a),
-        n0_mult = n0_mult, w_inf = params@species_params$w_inf, a=a, n=n, q=q))
-    #set densities at w > w_inf to 0
-    initial_n[unlist(tapply(params@w,1:no_w,function(wx,w_inf) w_inf<wx, w_inf=params@species_params$w_inf))] <- 0
-    # Also any densities at w < w_min set to 0
-    initial_n[unlist(tapply(params@w,1:no_w,function(wx,w_min)w_min>wx, w_min=params@species_params$w_min))] <- 0    
-    return(initial_n)
-}
+    gear_names <- dimnames(params@catchability)[[1]]
+    if (!all(gear_names %in% dimnames(effort)[[2]])) {
+        stop("Gear names in the MizerParams object (", 
+             paste(gear_names, collapse = ", "), 
+             ") do not match those in the effort array.")
+    }
+    # Sort effort array to match order in MizerParams
+    effort <- effort[, gear_names, drop = FALSE]
+    
+    if (is.null(dimnames(effort)[[1]])) {
+        stop("The time dimname of the effort argument must be numeric.")
+    }
+    time_effort <- as.numeric(dimnames(effort)[[1]])
+    if (any(is.na(time_effort))) {
+        stop("The time dimname of the effort argument must be numeric.")
+    }
+    if (is.unsorted(time_effort)) {
+        stop("The time dimname of the effort argument should be increasing.")
+    }
+    effort
+}
\ No newline at end of file
