@@ -15,16 +15,16 @@
 #'   \itemize{
 #'     \item encounter from [mizerEncounter()]
 #'     \item feeding_level from [mizerFeedingLevel()]
+#'     \item e from [mizerEReproAndGrowth()]
+#'     \item e_repro from [mizerERepro()]
+#'     \item e_growth from [mizerEGrowth()]
 #'     \item pred_rate from [mizerPredRate()]
 #'     \item pred_mort from [mizerPredMort()]
 #'     \item f_mort from [mizerFMort()]
 #'     \item mort from [mizerMort()]
-#'     \item resource_mort from [mizerResourceMort()]
-#'     \item e from [mizerEReproAndGrowth()]
-#'     \item e_repro from [mizerERepro()]
-#'     \item e_growth from [mizerEGrowth()]
 #'     \item rdi from [mizerRDI()]
 #'     \item rdd from [BevertonHoltRDD()]
+#'     \item resource_mort from [mizerResourceMort()]
 #'   }
 #' However you can replace any of these rate functions by your own rate
 #' function if you wish, see [setRateFunction()] for details.
@@ -47,6 +47,7 @@ mizerRates <- function(params, n, n_pp, n_other,
                        t = 0, effort, rates_fns, ...) {
     r <- list()
     
+    ## Growth ----
     # Calculate rate E_{e,i}(w) of encountered food
     r$encounter <- rates_fns$Encounter(
         params, n = n, n_pp = n_pp, n_other = n_other, t = t, ...)
@@ -58,33 +59,6 @@ mizerRates <- function(params, n, n_pp, n_other,
     r$e <- rates_fns$EReproAndGrowth(
         params, n = n, n_pp = n_pp, n_other = n_other,
         encounter = r$encounter, feeding_level = r$feeding_level, t = t, ...)
-    
-    ## Calculate all mortalities, before negative e values are turned into zero
-    #  in getERepro() and getEGrowth() functions
-    # Calculate the predation rate
-    r$pred_rate <- rates_fns$PredRate(
-        params, n = n, n_pp = n_pp, n_other = n_other,
-        feeding_level = r$feeding_level, t = t, ...)
-    # Calculate predation mortality on fish \mu_{p,i}(w)
-    r$pred_mort <- rates_fns$PredMort(
-        params, n = n, n_pp = n_pp, n_other = n_other,
-        pred_rate = r$pred_rate, t = t, ...)
-    
-    # Calculate mortality on the resource spectrum
-    r$resource_mort <- rates_fns$ResourceMort(
-        params, n = n, n_pp = n_pp, n_other = n_other,
-        pred_rate = r$pred_rate, t = t, ...)
-    
-    # Calculate fishing mortality
-    r$f_mort <- rates_fns$FMort(
-        params, n = n, n_pp = n_pp, n_other = n_other, 
-        effort = effort, t = t, ...)
-    # Calculate total mortality \mu_i(w)
-    r$mort <- rates_fns$Mort(
-        params, n = n, n_pp = n_pp, n_other = n_other,
-        f_mort = r$f_mort, pred_mort = r$pred_mort, t = t, ...)
-    
-    ## Now calculate energy for growth and reproduction
     # Calculate the energy for reproduction
     r$e_repro <- rates_fns$ERepro(
         params, n = n, n_pp = n_pp, n_other = n_other, 
@@ -94,16 +68,42 @@ mizerRates <- function(params, n, n_pp, n_other,
         params, n = n, n_pp = n_pp, n_other = n_other,
         e_repro = r$e_repro, e = r$e, t = t, ...)
     
-    # R_{p,i}
+    ## Mortality ----
+    # Calculate the predation rate
+    r$pred_rate <- rates_fns$PredRate(
+        params, n = n, n_pp = n_pp, n_other = n_other,
+        feeding_level = r$feeding_level, t = t, ...)
+    # Calculate predation mortality on fish \mu_{p,i}(w)
+    r$pred_mort <- rates_fns$PredMort(
+        params, n = n, n_pp = n_pp, n_other = n_other,
+        pred_rate = r$pred_rate, t = t, ...)
+    # Calculate fishing mortality
+    r$f_mort <- rates_fns$FMort(
+        params, n = n, n_pp = n_pp, n_other = n_other, 
+        effort = effort, t = t, 
+        e_growth = r$e_growth, pred_mort = r$pred_mort, ...)
+    # Calculate total mortality \mu_i(w)
+    r$mort <- rates_fns$Mort(
+        params, n = n, n_pp = n_pp, n_other = n_other,
+        f_mort = r$f_mort, pred_mort = r$pred_mort, t = t, ...)
+    
+    ## Reproduction ----
+    # R_di
     r$rdi <- rates_fns$RDI(
         params, n = n, n_pp = n_pp, n_other = n_other,
         e_growth = r$e_growth,
         mort = r$mort,
         e_repro = r$e_repro, t = t, ...)
-    # R_i
+    # R_dd
     r$rdd <- rates_fns$RDD(
         rdi = r$rdi, species_params = params@species_params, ...)
     
+    ## Resource ----
+    # Calculate mortality on the resource spectrum
+    r$resource_mort <- rates_fns$ResourceMort(
+        params, n = n, n_pp = n_pp, n_other = n_other,
+        pred_rate = r$pred_rate, t = t, ...)
+    
     return(r)
 }
 
@@ -161,7 +161,6 @@ mizerRates <- function(params, n, n_pp, n_other,
 #' same arguments.
 #' 
 #' @inheritParams mizerRates
-#' @param t The current time. Unused
 #' @param ... Unused
 #'   
 #' @return A named two dimensional array (predator species x predator size) with
@@ -224,7 +223,6 @@ mizerEncounter <- function(params, n, n_pp, n_other, t, ...) {
         
         encounter <- params@search_vol * avail_energy
     }
-    dimnames(encounter) <- dimnames(params@metab)
     
     # Add contributions from other components
     for (i in seq_along(params@other_encounter)) {
@@ -234,11 +232,9 @@ mizerEncounter <- function(params, n, n_pp, n_other, t, ...) {
                          n = n, n_pp = n_pp, n_other = n_other,
                          component = names(params@other_encounter)[[i]], ...))
     }
-    
     return(encounter)
 }
 
-
 #' Get feeding level needed to project standard mizer model
 #' 
 #' You would not usually call this function directly but instead use
@@ -286,7 +282,6 @@ mizerFeedingLevel <- function(params, n, n_pp, n_other, t, encounter, ...) {
     return(encounter / (encounter + params@intake_max))
 }
 
-
 #' Get energy rate available for reproduction and growth  needed to project 
 #' standard mizer model
 #'
@@ -345,259 +340,6 @@ mizerEReproAndGrowth <- function(params, n, n_pp, n_other, t, encounter,
         params@metab
 }
 
-
-#' Get predation rate needed to project standard mizer model
-#' 
-#' Calculates the potential rate (in units 1/year) at which a prey individual of
-#' a given size \eqn{w} is killed by predators from species \eqn{j}. In formulas
-#' \deqn{{\tt pred\_rate}_j(w_p) = \int \phi_j(w,w_p) (1-f_j(w)) 
-#'   \gamma_j(w) N_j(w) \, dw.}{pred_rate_j(w_p) = \int\phi_i(w,w_p) (1-f_i(w)) 
-#'   \gamma_i(w) N_i(w) dw.}
-#' This potential rate is used in the function [mizerPredMort()] to
-#' calculate the realised predation mortality rate on the prey individual.
-#' You would not usually call this
-#' function directly but instead use [getPredRate()], which then calls this
-#' function unless an alternative function has been registered, see below.
-#' 
-#' @section Your own predation rate function:
-#' By default [getPredRate()] calls [mizerPredRate()]. However you can
-#' replace this with your own alternative predation rate function. If 
-#' your function is called `"myPredRate"` then you register it in a MizerParams
-#' object `params` with
-#' ```
-#' params <- setRateFunction(params, "PredRate", "myPredRate")
-#' ```
-#' Your function will then be called instead of [mizerPredRate()], with
-#' the same arguments.
-#'
-#' @inheritParams mizerEReproAndGrowth
-#'   
-#' @return A named two dimensional array (predator species x prey size) with the
-#'   predation rate, where the prey size runs over fish community plus resource
-#'   spectrum.
-#' @export
-#' @family mizer rate functions
-mizerPredRate <- function(params, n, n_pp, n_other, t, feeding_level, ...) {
-    no_sp <- dim(params@interaction)[1]
-    no_w <- length(params@w)
-    no_w_full <- length(params@w_full)
-    
-    # If the feeding kernel does not have a fixed predator/prey mass ratio
-    # then the integral is not a convolution integral and we can not use fft.
-    # In this case we use the code from mizer version 0.3
-    if (length(params@ft_pred_kernel_p) == 1) {
-        n_total_in_size_bins <- sweep(n, 2, params@dw, '*', check.margin = FALSE)
-        # The next line is a bottle neck
-        pred_rate <- sweep(params@pred_kernel, c(1,2),
-                           (1 - feeding_level) * params@search_vol * 
-                               n_total_in_size_bins,
-                           "*", check.margin = FALSE)
-        # integrate over all predator sizes
-        pred_rate <- colSums(aperm(pred_rate, c(2, 1, 3)), dims = 1)
-        return(pred_rate)
-    }
-
-    # Get indices of w_full that give w
-    idx_sp <- (no_w_full - no_w + 1):no_w_full
-    # We express the result as a a convolution  involving
-    # two objects: Q[i,] and ft_pred_kernel_p[i,].
-    # Here Q[i,] is all the integrand of (3.12) except the feeding kernel
-    # and theta
-    Q <- matrix(0, nrow = no_sp, ncol = no_w_full)
-    # We fill the end of each row of Q with the proper values
-    Q[, idx_sp] <- sweep( (1 - feeding_level) * params@search_vol * n, 2,
-                         params@dw, "*")
-
-    # We do our spectral integration in parallel over the different species
-    pred_rate <- Re(t(mvfft(t(params@ft_pred_kernel_p) *
-                                 mvfft(t(Q)), inverse = TRUE))) / no_w_full
-    # Due to numerical errors we might get negative or very small entries that
-    # should be 0
-    pred_rate[pred_rate < 1e-18] <- 0
-    
-    dimnames(pred_rate) <- list(sp = params@species_params$species,
-                                w_prey = names(n_pp))
-    return(pred_rate * params@ft_mask)
-}
-
-
-#' Get total predation mortality rate needed to project standard mizer model
-#'
-#' Calculates the total predation mortality rate \eqn{\mu_{p,i}(w_p)} (in units
-#' of 1/year) on each prey species by prey size:
-#' \deqn{\mu_{p.i}(w_p) = \sum_j {\tt pred\_rate}_j(w_p)\, \theta_{ji}.}{
-#'   \mu_{p.i}(w_p) = \sum_j pred_rate_j(w_p) \theta_{ji}.}
-#' You would not usually call this
-#' function directly but instead use [getPredMort()], which then calls this
-#' function unless an alternative function has been registered, see below.
-#' 
-#' @section Your own predation mortality function:
-#' By default [getPredMort()] calls [mizerPredMort()]. However you can
-#' replace this with your own alternative predation mortality function. If 
-#' your function is called `"myPredMort"` then you register it in a MizerParams
-#' object `params` with
-#' ```
-#' params <- setRateFunction(params, "PredMort", "myPredMort")
-#' ```
-#' Your function will then be called instead of [mizerPredMort()], with the
-#' same arguments.
-#' 
-#' @inheritParams mizerRates
-#' @param pred_rate A two dimensional array (predator species x predator size)
-#'   with the feeding level.
-#'
-#' @return A two dimensional array (prey species x prey size) with the predation
-#'   mortality
-#' @family mizer rate functions
-#' @export
-mizerPredMort <- function(params, n, n_pp, n_other, t, pred_rate, ...) {
-    idx_sp <- (length(params@w_full) - 
-                   length(params@w) + 1):length(params@w_full)
-    return((base::t(params@interaction) %*% pred_rate)[, idx_sp, drop = FALSE])
-}
-
-#' Get predation mortality rate for resource needed to project standard mizer 
-#' model
-#' 
-#' Calculates the predation mortality rate \eqn{\mu_p(w)} on the resource
-#' spectrum by resource size (in units 1/year).
-#' You would not usually call this
-#' function directly but instead use [getResourceMort()], which then calls this
-#' function unless an alternative function has been registered, see below.
-#' 
-#' @section Your own resource mortality function:
-#' By default [getResourceMort()] calls [mizerResourceMort()]. However you can
-#' replace this with your own alternative resource mortality function. If 
-#' your function is called `"myResourceMort"` then you register it in a MizerParams
-#' object `params` with
-#' ```
-#' params <- setRateFunction(params, "ResourceMort", "myResourceMort")
-#' ```
-#' Your function will then be called instead of [mizerResourceMort()], with the
-#' same arguments.
-#' 
-#' @inheritParams mizerRates
-#' @param pred_rate A two dimensional array (predator species x prey size) with
-#'   the predation rate, where the prey size runs over fish community plus
-#'   resource spectrum.
-#'
-#' @return A vector of mortality rate by resource size.
-#' @family mizer rate functions
-#' @export
-mizerResourceMort <- 
-    function(params, n, n_pp, n_other, t, pred_rate, ...) {
-
-    return(as.vector(params@species_params$interaction_resource %*% pred_rate))
-}
-
-
-#' Get the fishing mortality by time, gear, species and size needed to project 
-#' standard mizer model
-#'
-#' Calculates the fishing mortality rate \eqn{F_{g,i,w}} by gear, species and
-#' size at each time step in the `effort` argument (in units 1/year).
-#' This is a helper function for [mizerFMort()].
-#' 
-#' @inheritParams mizerRates
-#' @param effort A vector with the effort for each fishing gear.
-#'   
-#' @return An three dimensional array (gear x species x size) with the
-#'    fishing mortality 
-#' @note Here: fishing mortality = catchability x selectivity x effort.
-#' 
-#' @export
-#' @family mizer rate functions
-mizerFMortGear <- function(params, effort) {
-    # Streamlined for speed increase - note use of recycling
-    out <- params@selectivity
-    out[] <- effort * c(params@catchability) * c(params@selectivity)
-    return(out)
-}
-
-
-#' Get the total fishing mortality rate from all fishing gears by time, species
-#' and size needed to project standard mizer model
-#' 
-#' Calculates the total fishing mortality  (in units 1/year) from all gears by
-#' species and size at each time step in the `effort` argument.
-#' The total fishing mortality is just the sum of the fishing mortalities
-#' imposed by each gear, \eqn{\mu_{f.i}(w)=\sum_g F_{g,i,w}}.
-#' You would not usually call this
-#' function directly but instead use [getFMort()], which then calls this
-#' function unless an alternative function has been registered, see below.
-#' 
-#' @section Your own fishing mortality function:
-#' By default [getFMort()] calls [mizerFMort()]. However you can
-#' replace this with your own alternative fishing mortality function. If 
-#' your function is called `"myFMort"` then you register it in a MizerParams
-#' object `params` with
-#' ```
-#' params <- setRateFunction(params, "FMort", "myFMort")
-#' ```
-#' Your function will then be called instead of [mizerFMort()], with the
-#' same arguments.
-#' 
-#' @inheritParams mizerRates
-#' @param effort A vector with the effort for each fishing gear.
-#'
-#' @return An array (species x size) with the fishing mortality.
-#' @note Here: fishing mortality = catchability x selectivity x effort.
-#' @export
-#' @family mizer rate functions
-mizerFMort <- function(params, effort, ...) {
-    colSums(mizerFMortGear(params, effort))
-}
-
-#' Get total mortality rate needed to project standard mizer model
-#'
-#' Calculates the total mortality rate \eqn{\mu_i(w)}  (in units 1/year) on each
-#' species by size from predation mortality, background mortality and fishing
-#' mortality.
-#' You would not usually call this
-#' function directly but instead use [getMort()], which then calls this
-#' function unless an alternative function has been registered, see below.
-#' 
-#' If your model contains additional components that you added with 
-#' [setComponent()] and for which you specified a `mort_fun` function then
-#' the mortality inflicted by these components will be included in the returned
-#' value.
-#' 
-#' @section Your own mortality function:
-#' By default [getMort()] calls [mizerMort()]. However you can
-#' replace this with your own alternative mortality function. If 
-#' your function is called `"myMort"` then you register it in a MizerParams
-#' object `params` with
-#' ```
-#' params <- setRateFunction(params, "Mort", "myMort")
-#' ```
-#' Your function will then be called instead of [mizerMort()], with the
-#' same arguments.
-#'
-#' @inheritParams mizerRates
-#' @param f_mort A two dimensional array (species x size) with the fishing
-#'   mortality
-#' @param pred_mort A two dimensional array (species x size) with the predation
-#'   mortality
-#' @param ... Unused
-#'
-#' @return A named two dimensional array (species x size) with the total
-#'   mortality rates.
-#' @export
-#' @family mizer rate functions
-mizerMort <- function(params, n, n_pp, n_other, t, f_mort, pred_mort, ...){
-    mort <- pred_mort + params@mu_b + f_mort
-    # Add contributions from other components
-    for (i in seq_along(params@other_mort)) {
-        mort <- mort + 
-            do.call(params@other_mort[[i]], 
-                    list(params = params,
-                         n = n, n_pp = n_pp, n_other = n_other, t = t,
-                         component = names(params@other_mort)[[i]], ...))
-    }
-    return(mort)
-}
-
-
 #' Get energy rate available for reproduction needed to project standard mizer 
 #' model
 #'
@@ -678,18 +420,243 @@ mizerEGrowth <- function(params, n, n_pp, n_other, t, e_repro, e, ...) {
 }
 
 
+#' Get predation rate needed to project standard mizer model
+#' 
+#' Calculates the potential rate (in units 1/year) at which a prey individual of
+#' a given size \eqn{w} is killed by predators from species \eqn{j}. In formulas
+#' \deqn{{\tt pred\_rate}_j(w_p) = \int \phi_j(w,w_p) (1-f_j(w)) 
+#'   \gamma_j(w) N_j(w) \, dw.}{pred_rate_j(w_p) = \int\phi_i(w,w_p) (1-f_i(w)) 
+#'   \gamma_i(w) N_i(w) dw.}
+#' This potential rate is used in the function [mizerPredMort()] to
+#' calculate the realised predation mortality rate on the prey individual.
+#' You would not usually call this
+#' function directly but instead use [getPredRate()], which then calls this
+#' function unless an alternative function has been registered, see below.
+#' 
+#' @section Your own predation rate function:
+#' By default [getPredRate()] calls [mizerPredRate()]. However you can
+#' replace this with your own alternative predation rate function. If 
+#' your function is called `"myPredRate"` then you register it in a MizerParams
+#' object `params` with
+#' ```
+#' params <- setRateFunction(params, "PredRate", "myPredRate")
+#' ```
+#' Your function will then be called instead of [mizerPredRate()], with
+#' the same arguments.
+#'
+#' @inheritParams mizerEReproAndGrowth
+#'   
+#' @return A named two dimensional array (predator species x prey size) with the
+#'   predation rate, where the prey size runs over fish community plus resource
+#'   spectrum.
+#' @export
+#' @family mizer rate functions
+mizerPredRate <- function(params, n, n_pp, n_other, t, feeding_level, ...) {
+    no_sp <- dim(params@interaction)[1]
+    no_w <- length(params@w)
+    no_w_full <- length(params@w_full)
+    
+    # If the feeding kernel does not have a fixed predator/prey mass ratio
+    # then the integral is not a convolution integral and we can not use fft.
+    # In this case we use the code from mizer version 0.3
+    if (length(params@ft_pred_kernel_p) == 1) {
+        n_total_in_size_bins <- sweep(n, 2, params@dw, '*', check.margin = FALSE)
+        # The next line is a bottle neck
+        pred_rate <- sweep(params@pred_kernel, c(1,2),
+                           (1 - feeding_level) * params@search_vol * 
+                               n_total_in_size_bins,
+                           "*", check.margin = FALSE)
+        # integrate over all predator sizes
+        pred_rate <- colSums(aperm(pred_rate, c(2, 1, 3)), dims = 1)
+        return(pred_rate)
+    }
+
+    # Get indices of w_full that give w
+    idx_sp <- (no_w_full - no_w + 1):no_w_full
+    # We express the result as a a convolution  involving
+    # two objects: Q[i,] and ft_pred_kernel_p[i,].
+    # Here Q[i,] is all the integrand of (3.12) except the feeding kernel
+    # and theta
+    Q <- matrix(0, nrow = no_sp, ncol = no_w_full)
+    # We fill the end of each row of Q with the proper values
+    Q[, idx_sp] <- sweep( (1 - feeding_level) * params@search_vol * n, 2,
+                         params@dw, "*")
+
+    # We do our spectral integration in parallel over the different species
+    pred_rate <- Re(t(mvfft(t(params@ft_pred_kernel_p) *
+                                 mvfft(t(Q)), inverse = TRUE))) / no_w_full
+    # Due to numerical errors we might get negative or very small entries that
+    # should be 0
+    pred_rate[pred_rate < 1e-18] <- 0
+    
+    return(pred_rate * params@ft_mask)
+}
+
+
+#' Get total predation mortality rate needed to project standard mizer model
+#'
+#' Calculates the total predation mortality rate \eqn{\mu_{p,i}(w_p)} (in units
+#' of 1/year) on each prey species by prey size:
+#' \deqn{\mu_{p.i}(w_p) = \sum_j {\tt pred\_rate}_j(w_p)\, \theta_{ji}.}{
+#'   \mu_{p.i}(w_p) = \sum_j pred_rate_j(w_p) \theta_{ji}.}
+#' You would not usually call this
+#' function directly but instead use [getPredMort()], which then calls this
+#' function unless an alternative function has been registered, see below.
+#' 
+#' @section Your own predation mortality function:
+#' By default [getPredMort()] calls [mizerPredMort()]. However you can
+#' replace this with your own alternative predation mortality function. If 
+#' your function is called `"myPredMort"` then you register it in a MizerParams
+#' object `params` with
+#' ```
+#' params <- setRateFunction(params, "PredMort", "myPredMort")
+#' ```
+#' Your function will then be called instead of [mizerPredMort()], with the
+#' same arguments.
+#' 
+#' @inheritParams mizerRates
+#' @param pred_rate A two dimensional array (predator species x predator size)
+#'   with the feeding level.
+#'
+#' @return A two dimensional array (prey species x prey size) with the predation
+#'   mortality
+#' @family mizer rate functions
+#' @export
+mizerPredMort <- function(params, n, n_pp, n_other, t, pred_rate, ...) {
+    idx_sp <- (length(params@w_full) - 
+                   length(params@w) + 1):length(params@w_full)
+    return((base::t(params@interaction) %*% pred_rate)[, idx_sp, drop = FALSE])
+}
+
+#' Get the fishing mortality by time, gear, species and size needed to project 
+#' standard mizer model
+#'
+#' Calculates the fishing mortality rate \eqn{F_{g,i,w}} by gear, species and
+#' size at each time step in the `effort` argument (in units 1/year).
+#' This is a helper function for [mizerFMort()].
+#' 
+#' @inheritParams mizerRates
+#' @param effort A vector with the effort for each fishing gear.
+#'   
+#' @return An three dimensional array (gear x species x size) with the
+#'    fishing mortality 
+#' @note Here: fishing mortality = catchability x selectivity x effort.
+#' 
+#' @export
+#' @family mizer rate functions
+mizerFMortGear <- function(params, effort) {
+    # Streamlined for speed increase - note use of recycling
+    out <- params@selectivity
+    out[] <- effort * c(params@catchability) * c(params@selectivity)
+    return(out)
+}
+
+
+#' Get the total fishing mortality rate from all fishing gears by time, species
+#' and size needed to project standard mizer model
+#' 
+#' Calculates the total fishing mortality  (in units 1/year) from all gears by
+#' species and size at each time step in the `effort` argument.
+#' The total fishing mortality is just the sum of the fishing mortalities
+#' imposed by each gear, \eqn{\mu_{f.i}(w)=\sum_g F_{g,i,w}}.
+#' You would not usually call this
+#' function directly but instead use [getFMort()], which then calls this
+#' function unless an alternative function has been registered, see below.
+#' 
+#' @section Your own fishing mortality function:
+#' By default [getFMort()] calls [mizerFMort()]. However you can
+#' replace this with your own alternative fishing mortality function. If 
+#' your function is called `"myFMort"` then you register it in a MizerParams
+#' object `params` with
+#' ```
+#' params <- setRateFunction(params, "FMort", "myFMort")
+#' ```
+#' Your function will then be called instead of [mizerFMort()], with the
+#' same arguments.
+#' 
+#' @inheritParams mizerRates
+#' @param effort A vector with the effort for each fishing gear.
+#' @param e_growth An array (species x size) with the energy available for
+#'   growth as calculated by [getEGrowth()]. Unused.
+#' @param pred_mort A two dimensional array (species x size) with the predation
+#'   mortality as calculated by [getPredMort()]. Unused.
+#'
+#' @return An array (species x size) with the fishing mortality.
+#' @note Here: fishing mortality = catchability x selectivity x effort.
+#' @export
+#' @family mizer rate functions
+mizerFMort <- function(params, n, n_pp, n_other, t, effort,
+                       e_growth, pred_mort, ...) {
+    colSums(mizerFMortGear(params, effort))
+}
+
+#' Get total mortality rate needed to project standard mizer model
+#'
+#' Calculates the total mortality rate \eqn{\mu_i(w)}  (in units 1/year) on each
+#' species by size from predation mortality, background mortality and fishing
+#' mortality.
+#' You would not usually call this
+#' function directly but instead use [getMort()], which then calls this
+#' function unless an alternative function has been registered, see below.
+#' 
+#' If your model contains additional components that you added with 
+#' [setComponent()] and for which you specified a `mort_fun` function then
+#' the mortality inflicted by these components will be included in the returned
+#' value.
+#' 
+#' @section Your own mortality function:
+#' By default [getMort()] calls [mizerMort()]. However you can
+#' replace this with your own alternative mortality function. If 
+#' your function is called `"myMort"` then you register it in a MizerParams
+#' object `params` with
+#' ```
+#' params <- setRateFunction(params, "Mort", "myMort")
+#' ```
+#' Your function will then be called instead of [mizerMort()], with the
+#' same arguments.
+#'
+#' @inheritParams mizerRates
+#' @param f_mort A two dimensional array (species x size) with the fishing
+#'   mortality
+#' @param pred_mort A two dimensional array (species x size) with the predation
+#'   mortality
+#' @param ... Unused
+#'
+#' @return A named two dimensional array (species x size) with the total
+#'   mortality rates.
+#' @export
+#' @family mizer rate functions
+mizerMort <- function(params, n, n_pp, n_other, t, f_mort, pred_mort, ...){
+    mort <- pred_mort + params@mu_b + f_mort
+    # Add contributions from other components
+    for (i in seq_along(params@other_mort)) {
+        mort <- mort + 
+            do.call(params@other_mort[[i]], 
+                    list(params = params,
+                         n = n, n_pp = n_pp, n_other = n_other, t = t,
+                         component = names(params@other_mort)[[i]], ...))
+    }
+    return(mort)
+}
+
+
 #' Get density-independent rate of reproduction needed to project standard
 #' mizer model
 #'
-#' Calculates the density-independent rate of total egg production \eqn{R_{p.i}}
-#' (units 1/year) before density dependence, by species. This is obtained by
-#' taking the total rate at which energy is invested in reproduction,
-#' multiplying by the reproductive efficiency `erepro` and dividing by the egg
-#' size `w_min`, and by a factor of two to account for the two sexes.
+#' Calculates the density-independent rate of total egg production 
+#' \eqn{R_{di}}{R_di} (units 1/year) before density dependence, by species. 
 #' You would not usually call this
 #' function directly but instead use [getRDI()], which then calls this
 #' function unless an alternative function has been registered, see below.
 #' 
+#' This rate is obtained by taking the per capita rate \eqn{E_r(w)\psi(w)} at
+#' which energy is invested in reproduction, as calculated by [getERepro()],
+#' multiplying it by the number of individuals\eqn{N(w)} and integrating over all sizes
+#' \eqn{w} and then multiplying by the reproductive efficiency \eqn{\epsilon}
+#' and dividing by the egg size `w_min`, and by a factor of two to account for
+#' the two sexes:
+#' \deqn{R_{di} = \frac{\epsilon}{2 w_{min}} \int N(w)  E_r(w) \psi(w) \, dw}{R_di = (\epsilon/(2 w_min)) \int N(w)  E_r(w) \psi(w) dw}
+#' 
 #' Used by [getRDD()] to calculate the actual, density dependent rate.
 #' See [setReproduction()] for more details.
 #' 
@@ -728,11 +695,15 @@ mizerRDI <- function(params, n, n_pp, n_other, t,
 
 #' Beverton Holt function to calculate density-dependent reproduction rate
 #'
-#' Takes the density-independent rates \eqn{R_p} of egg production and returns
-#' reduced, density-dependent reproduction rates \eqn{R} given as \deqn{R = R_p
-#' \frac{R_{max}}{R_p + R_{max}}}{R = R_p R_{max}/(R_p + R_{max})} where
-#' \eqn{R_{max}} are the maximum possible reproduction rates that must be
+#' Takes the density-independent rates \eqn{R_{di}}{R_di} of egg production (as
+#' calculated by [getRDI()]) and returns
+#' reduced, density-dependent reproduction rates \eqn{R_{dd}}{R_dd} given as 
+#' \deqn{R_{dd} = R_{di}
+#' \frac{R_{max}}{R_{di} + R_{max}}}{R_dd = R_di R_max/(R_di + R_max)} where
+#' \eqn{R_{max}}{R_max} are the maximum possible reproduction rates that must be
 #' specified in a column in the species parameter dataframe.
+#' (All quantities in the above equation are species-specific but we dropped
+#' the species index for simplicity.)
 #'
 #' This is only one example of a density-dependence. You can write your own
 #' function based on this example, returning different density-dependent
@@ -740,10 +711,10 @@ mizerRDI <- function(params, n, n_pp, n_other, t,
 #' and [SheperdRDD()]. For more explanation see
 #' [setReproduction()].
 #'
-#' @param rdi Vector of density-independent reproduction rates \eqn{R_p} for all
-#'   species.
+#' @param rdi Vector of density-independent reproduction rates 
+#'   \eqn{R_{di}}{R_di} for all species.
 #' @param species_params A species parameter dataframe. Must contain a column
-#'   R_max holding the maximum reproduction rate \eqn{R_{max}} for each species.
+#'   R_max holding the maximum reproduction rate \eqn{R_{max}}{R_max} for each species.
 #' @param ... Unused
 #'
 #' @return Vector of density-dependent reproduction rates.
@@ -758,12 +729,12 @@ BevertonHoltRDD <- function(rdi, species_params, ...) {
 
 #' Ricker function to calculate density-dependent reproduction rate
 #' 
-#' Takes the density-independent rates \eqn{R_p} of egg production and returns
-#' reduced, density-dependent rates \eqn{R} given as
-#' \deqn{R = R_p \exp{- b R_p}}
+#' Takes the density-independent rates \eqn{R_{di}}{R_di} of egg production and returns
+#' reduced, density-dependent rates \eqn{R_{dd}}{R_dd} given as
+#' \deqn{R_{dd} = R_{di} \exp{- b R_{di}}}{R_dd = R_di \exp{- b R_di}}
 #' 
-#' @param rdi Vector of density-independent reproduction rates \eqn{R_p} for
-#'   all species.
+#' @param rdi Vector of density-independent reproduction rates 
+#'   \eqn{R_{di}}{R_di} for all species.
 #' @param species_params A species parameter dataframe. Must contain a column
 #'   `ricker_b` holding the coefficient b.
 #' @param ... Unused
@@ -780,12 +751,12 @@ RickerRDD <- function(rdi, species_params, ...) {
 
 #' Sheperd function to calculate density-dependent reproduction rate
 #' 
-#' Takes the density-independent rates \eqn{R_p} of egg production and returns
-#' reduced, density-dependent rates \eqn{R} as
-#' \deqn{R = \frac{R_p}{1+(b\ R_p)^c}}{R = R_p / (1 + (b R_p)^c)}
+#' Takes the density-independent rates \eqn{R_{di}}{R_di} of egg production and returns
+#' reduced, density-dependent rates \eqn{R_{dd}}{R_dd} given as
+#' \deqn{R_{dd} = \frac{R_{di}}{1+(b\ R_{di})^c}}{R_dd = R_di / (1 + (b R_di)^c)}
 #' 
-#' @param rdi Vector of density-independent reproduction rates \eqn{R_p} for
-#'   all species.
+#' @param rdi Vector of density-independent reproduction rates 
+#'   \eqn{R_{di}}{R_di} for all species.
 #' @param species_params A species parameter dataframe. Must contain columns
 #'   `sheperd_b` and `sheperd_c` with the parameters b and c.
 #' @param ... Unused
@@ -804,8 +775,8 @@ SheperdRDD <- function(rdi, species_params, ...) {
 #' 
 #' Simply returns its `rdi` argument.
 #' 
-#' @param rdi Vector of density-independent reproduction rates \eqn{R_p} for
-#'   all species.
+#' @param rdi Vector of density-independent reproduction rates 
+#'   \eqn{R_{di}}{R_di} for all species.
 #' @param ... Not used.
 #' 
 #' @return Vector of density-dependent reproduction rates.
@@ -815,13 +786,12 @@ noRDD <- function(rdi, ...) {
     return(rdi)
 }
 
-
 #' Give constant reproduction rate
 #'
 #' Simply returns the value from `species_params$constant_reproduction`.
 #'
-#' @param rdi Vector of density-independent reproduction rates \eqn{R_p} for all
-#'   species.
+#' @param rdi Vector of density-independent reproduction rates 
+#'   \eqn{R_{di}}{R_di} for all species.
 #' @param species_params A species parameter dataframe. Must contain a column
 #'   `constant_reproduction`.
 #' @param ... Unused
@@ -832,3 +802,38 @@ noRDD <- function(rdi, ...) {
 constantRDD <- function(rdi, species_params, ...){
     return(species_params$constant_reproduction)
 }
+
+
+#' Get predation mortality rate for resource needed to project standard mizer 
+#' model
+#' 
+#' Calculates the predation mortality rate \eqn{\mu_p(w)} on the resource
+#' spectrum by resource size (in units 1/year).
+#' You would not usually call this
+#' function directly but instead use [getResourceMort()], which then calls this
+#' function unless an alternative function has been registered, see below.
+#' 
+#' @section Your own resource mortality function:
+#' By default [getResourceMort()] calls [mizerResourceMort()]. However you can
+#' replace this with your own alternative resource mortality function. If 
+#' your function is called `"myResourceMort"` then you register it in a MizerParams
+#' object `params` with
+#' ```
+#' params <- setRateFunction(params, "ResourceMort", "myResourceMort")
+#' ```
+#' Your function will then be called instead of [mizerResourceMort()], with the
+#' same arguments.
+#' 
+#' @inheritParams mizerRates
+#' @param pred_rate A two dimensional array (predator species x prey size) with
+#'   the predation rate, where the prey size runs over fish community plus
+#'   resource spectrum.
+#'
+#' @return A vector of mortality rate by resource size.
+#' @family mizer rate functions
+#' @export
+mizerResourceMort <- 
+    function(params, n, n_pp, n_other, t, pred_rate, ...) {
+        
+        return(as.vector(params@species_params$interaction_resource %*% pred_rate))
+    }
\ No newline at end of file
