@@ -76,16 +76,15 @@
 #' ## Set up a MizerParams object
 #' params <- newMultispeciesParams(NS_species_params_gears, inter)
 #' 
-#' ## If you change predation kernel parameters after setting up a model, you
-#' # need to call setPredKernel
-#' params@species_params["Cod", "beta"] <- 200
-#' params <- setPredKernel(params)
+#' ## If you change predation kernel parameters after setting up a model, 
+#' #  this will be used to recalculate the kernel
+#' species_params(params)["Cod", "beta"] <- 200
 #' 
 #' ## You can change to a different predation kernel type
-#' params@species_params$pred_kernel_type <- "box"
-#' params@species_params$ppmr_min <- 2
-#' params@species_params$ppmr_max <- 4
-#' params <- setPredKernel(params)
+#' species_params(params)$ppmr_max <- 4000
+#' species_params(params)$ppmr_min <- 200
+#' species_params(params)$pred_kernel_type <- "box"
+#' plot(w_full(params), getPredKernel(params)["Cod", 100, ], type="l", log="x")
 #' 
 #' ## If you need a kernel that depends also on prey size you need to define
 #' # it yourself.
@@ -140,6 +139,8 @@ setPredKernel <- function(params,
     # The smallest predator/prey mass ratio is 1
     ppmr <- params@w_full / params@w_full[1]
     phis <- get_phi(species_params, ppmr)
+    # Do not allow feeding at own size
+    phis[, 1] <- 0
     for (i in 1:no_sp) {
         phi <- phis[i, ]
         # Fourier transform of feeding kernel for evaluating available energy
@@ -180,6 +181,8 @@ getPredKernel <- function(params) {
     # The smallest predator/prey mass ratio is 1
     ppmr <- params@w_full / params@w_full[1]
     phis <- get_phi(species_params, ppmr)
+    # Do not allow feeding at own size
+    phis[, 1] <- 0
     pred_kernel <- 
         array(0,
               dim = c(no_sp, no_w, no_w_full),
