@@ -22,7 +22,7 @@ server <- function(input, output, session) {
     sim_old <- readRDS(file="hake_mullet.RDS")
     # Therefore we do not need the following:
     # p_bg  <- reactive({
-    #     setBackground(set_scaling_model(no_sp = input$no_sp, no_w = 400,
+    #     markBackground(set_scaling_model(no_sp = input$no_sp, no_w = 400,
     #                     min_w_inf = 10, max_w_inf = 1e5,
     #                     min_egg = 1e-4, min_w_mat = 10^(0.4),
     #                     knife_edge_size = Inf, kappa = 10000,
@@ -104,7 +104,7 @@ server <- function(input, output, session) {
     #     on.exit(progress$close())
     #     
     #     project(p(), t_max = 50, t_save = 5, effort = fixed_effort, 
-    #             shiny_progress = progress)
+    #             progress_bar = progress)
     # })
     
     # Data frame for yield plot
@@ -132,36 +132,34 @@ server <- function(input, output, session) {
         no_t <- dim(sim_old@n)[1]
         p@initial_n <- sim_old@n[no_t, , ]
         p@initial_n_pp <- sim_old@n_pp[no_t, ]
-        p@species_params$r_max <- Inf
+        p@species_params$R_max <- Inf
         
         # Retune the values of erepro so that we get the correct level of
-        # recruitment without stock-recruitment relationship
+        # reproduction without density dependence
         mumu <- getZ(p, p@initial_n, p@initial_n_pp, effort = fixed_effort)
         gg <- getEGrowth(p, p@initial_n, p@initial_n_pp)
         rdi <- getRDI(p, p@initial_n, p@initial_n_pp)
         for (i in (1:no_sp)) {
-            gg0 <- gg[i, p@species_params$w_min_idx[i]]
-            mumu0 <- mumu[i, p@species_params$w_min_idx[i]]
-            DW <- p@dw[p@species_params$w_min_idx[i]]
+            gg0 <- gg[i, p@w_min_idx[i]]
+            mumu0 <- mumu[i, p@w_min_idx[i]]
+            DW <- p@dw[p@w_min_idx[i]]
             p@species_params$erepro[i] <- p@species_params$erepro[i] *
-                (p@initial_n[i, p@species_params$w_min_idx[i]] *
+                (p@initial_n[i, p@w_min_idx[i]] *
                      (gg0 + DW * mumu0)) / rdi[i]
         }
         
         # Set new gear for hake
-        a <- p@species_params["Hake", "a"]
-        b <- p@species_params["Hake", "b"]
         p@species_params["Hake", "l50"] <- input$l50_hake
         p@species_params["Hake", "l25"] <- input$l25_hake
         p@selectivity["sigmoid_gear", "Hake", ] <-
-            sigmoid_length(p@w, input$l25_hake, input$l50_hake, a, b)
+            sigmoid_length(p@w, input$l25_hake, input$l50_hake, 
+                           p@species_params["Hake", ])
         # Set new gear for mullet
-        a <- p@species_params["Mullet", "a"]
-        b <- p@species_params["Mullet", "b"]
         p@species_params["Mullet", "l50"] <- input$l50_mullet
         p@species_params["Mullet", "l50"] <- input$l25_mullet
         p@selectivity["sigmoid_gear", "Mullet", ] <-
-            sigmoid_length(p@w, input$l25_mullet, input$l50_mullet, a, b)
+            sigmoid_length(p@w, input$l25_mullet, input$l50_mullet, 
+                           p@species_params["Mullet", ])
         p
     })
     
@@ -172,7 +170,7 @@ server <- function(input, output, session) {
         on.exit(progress$close())
         
         project(params(), t_max = 15, t_save = 0.1, effort = fixed_effort, 
-                shiny_progress = progress)
+                progress_bar = progress)
     })
     
     # Plot yield ####
@@ -471,7 +469,7 @@ ui <- fluidPage(
                     p("This unpacks a bit of the underlying mizer model of 
                       multispecies dynamics.  The grey lines are background 
                       species, each with its own dynamics;  the green line is a 
-                      plankton resource spectrum without which the whole fish 
+                      resource resource spectrum without which the whole fish 
                       assemblage would go to extinction.  The target species 
                       both eat, and are eaten by, the background species."),
                     p("In 2018, the system is at the steady state under 
@@ -533,4 +531,4 @@ ui <- fluidPage(
     )  # end sidebarlayout
 )
 
-shinyApp(ui = ui, server = server) # this launches your app
\ No newline at end of file
+shinyApp(ui = ui, server = server) # this launches your app
